// This file defines the "New Command" dialog. This dialog is also used
// for editing current commands. The definition of the "Select application"
// dialog is also contained in this file.

#include <eikcfdlg.h>
#include <eikfile.rsg>
#include <eikcmbut.h>

#include "NewCommandDialog.h"

// Set up member variables.
CNewCommandDialog::CNewCommandDialog(CVoiceControlDialog& aParent,
									 TInt aResourceFileId,
									 TBool aEditingCommand)
	: CEikDialog(), iParent(aParent), iResourceFileId(aResourceFileId),
	  iEditingCommand(aEditingCommand)
	{
	}

CNewCommandDialog::~CNewCommandDialog()
	{
	// NB: We don't delete iVoiceCommand because the parent dialog has taken
	// ownership of it.
	}

// This function will be called before the dialog is displayed.
void CNewCommandDialog::PreLayoutDynInitL()
	{
	// If we're actually editing a current command, the buttons and the
	// state of the edit windows will be different.
	if (iEditingCommand)
		{
		iVoiceCommand = iParent.GetSelectedCommand();

		// Set up the CBA.
		if ((iVoiceCommand->iCommand == EVoiceCommandOpenDocument) ||
			(iVoiceCommand->iCommand == EVoiceCommandOpenApplication))
			{
			ButtonGroupContainer().SetCommandL(0, ENewCommandBrowse, KBrowseLabel);
			}
		else
			{
			ButtonGroupContainer().SetCommandL(0, ENewCommandBlank, KBlankLabel);
			}
		ButtonGroupContainer().SetCommandL(1, ENewCommandRecord, KRerecordLabel);
		ButtonGroupContainer().SetCommandL(2, ENewCommandBlank, KBlankLabel);
		ButtonGroupContainer().SetCommandL(3, ENewCommandCancel, KDoneLabel);

		// Set the dialog's title.
		_LIT(KEditCommandTitle, "Edit command");
		SetTitleL(KEditCommandTitle);
		
		// Set up the contents of the edit windows to be the details of
		// the current command.
		((CEikEdwin*) Control(ENewCommandNameEditor))->SetTextL(iVoiceCommand->iName);

		((CEikChoiceList*) Control(ENewCommandCommandChoiceList))->
			SetCurrentItem(iVoiceCommand->iCommand);

		((CEikEdwin*) Control(ENewCommandArgumentEditor))->SetTextL(iVoiceCommand->iArgument);
		}
	else
		{
		// Create the dialog's associated new voice command object.
		iVoiceCommand = new (ELeave) CVoiceCommand;

		// Update the CBA with this dialog's commands.
		ButtonGroupContainer().SetCommandL(0, ENewCommandBlank, KBlankLabel);
		ButtonGroupContainer().SetCommandL(1, ENewCommandRecord, KRecordLabel);
		ButtonGroupContainer().SetCommandL(2, ENewCommandBlank, KBlankLabel);
		ButtonGroupContainer().SetCommandL(3, ENewCommandCancel, KCancelLabel);
		}

	// Update the displey.
	ButtonGroupContainer().DrawNow();
	}

// Function called when the user tries to exit the dialog.
// We always allow this.
TBool CNewCommandDialog::OkToExitL(TInt /*aButtonId*/)
	{
	return ETrue;
	}

TKeyResponse CNewCommandDialog::OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
	{
	if (aType == EEventKey)
		{
		switch (aKeyEvent.iScanCode)
			{
			case EStdKeyDevice0:
				// The top button has been pressed. The only case where we're
				// interested in this is when the user wants to browse for a file.
				{
				// Get the currently selected action.
				TVoiceCommandId choice = (TVoiceCommandId)
					((CEikChoiceList*) Control(ENewCommandCommandChoiceList))->
					CurrentItem();

				// Get the current filename.
				CEikEdwin* edwin = (CEikEdwin*) Control(ENewCommandArgumentEditor);
				TFileName fileName;
				edwin->GetText(fileName);
				
				// The "File open" dialog crashes if the user just enters
				// a drive name, eg. "c:", so we have to check for this.
				_LIT(KColon, ":");
				_LIT(KBackslash, "\\");
				if (!fileName.Right(1).CompareF(KColon))
					fileName.Append(KBackslash);
				
				// Only show the "Browse" or "Select app" dialogs if the right control is
				// selected, and the right action is to be performed.
				if (edwin->IsFocused())
					{
					if (choice == EVoiceCommandOpenDocument)
						{
						// Create the dialog.
						CEikFileOpenDialog* dialog = new (ELeave) CEikFileOpenDialog(&fileName);
						// We allow the showing of system files because it's useful.
						dialog->SetShowSystem(ETrue);
						// Show the dialog. If the user doesn't hit "Cancel",
						// set the filename to be what was selected.
						if (dialog->ExecuteLD(R_EIK_DIALOG_FILE_OPEN))
							edwin->SetTextL(&fileName);
						}
					else if (choice == EVoiceCommandOpenApplication)
						{
						TFileName appPath;
						// Create the "Select application" dialog.
						CSelectApplicationDialog* dialog = new (ELeave) CSelectApplicationDialog(appPath);
						// Show the dialog. If the user doesn't hit "Cancel",
						// set the app's filename based on what was selected.
						if (dialog->ExecuteLD(R_SELECT_APPLICATION_DIALOG))
							edwin->SetTextL(&appPath);
						}
					}
				}
				break;

			case EStdKeyDevice1:
				{
				// The second CBA key has been pressed. We either want to
				// record a new command, or re-record an existing command.
				if (iEditingCommand)
					RerecordL();
				else
					RecordL();
				}
				break;

			case EStdKeyDevice2:
				break;

			case EStdKeyDevice3:
				// The bottom CBA key has been pressed. If we're not
				// recording, we want to cancel/close the dialog. If we are
				// recording, we want to stop.
				if (iState == KRecorderIdle)
					{
					if (iEditingCommand)
						ModifyCommandInListL();
					else
						delete iVoiceCommand;
					TryExitL(ENewCommandCancel);
					}
				else
					{
					StopRecordingL();
					}
				break;

			default:
				// If we didn't understand the event, pass it on to the
				// base class.
				CEikDialog::OfferKeyEventL(aKeyEvent, aType);
				break;
			}
		}
	// We must return this, or the key gets passed to the control panel!
	return EKeyWasConsumed;
	}

void CNewCommandDialog::HandleControlEventL(CCoeControl* aControl, TCoeEvent /*aEventType*/)
	{
	// If the user has changed the action to be performed by this command,
	// update the label for the "command argument" field.
	_LIT(KNameLabel, "Name");
	_LIT(KNumberLabel, "Number");
	_LIT(KPathLabel, "Path");

	if (aControl == Control(ENewCommandCommandChoiceList))
		{
		TBuf<10> newLabel;
		switch (((CEikChoiceList*) Control(ENewCommandCommandChoiceList))->CurrentItem())
			{
		case EVoiceCommandDialContact:
		case EVoiceCommandDialContactMobile:
			newLabel = KNameLabel;
			break;

		case EVoiceCommandDialNumber:
			newLabel = KNumberLabel;
			break;

		case EVoiceCommandOpenDocument:
		case EVoiceCommandOpenApplication:
			newLabel = KPathLabel;
			break;
			}
		ControlCaption(ENewCommandArgumentEditor)->SetTextL(newLabel);
		// Only redraw the caption to reduce flicker.
		ControlCaption(ENewCommandArgumentEditor)->DrawDeferred();
		}
	}

// This function will be called when the dialog's current line changes. We
// use it to decide whether or not to show a "Browse" button for choosing
// files, or a "Select" button for choosing apps.
void CNewCommandDialog::LineChangedL(TInt aControlId)
	{
	// Get the currently selected action.
	TVoiceCommandId choice = (TVoiceCommandId)
		((CEikChoiceList*) Control(ENewCommandCommandChoiceList))->CurrentItem();

	// We only want to display the "Browse" button if the user is currently
	// selecting the path for opening a file.
	if ((aControlId == ENewCommandArgumentEditor) &&
		(choice == EVoiceCommandOpenDocument))
		{
		ButtonGroupContainer().SetCommandL(0, ENewCommandBrowse, KBrowseLabel);
		// Only draw the one button to reduce flicker.
		ButtonGroupContainer().ButtonById(ENewCommandBrowse)->DrawNow();
		}
	// Only display the "Select" button if the user needs to select an app.
	else if ((aControlId == ENewCommandArgumentEditor) &&
		(choice == EVoiceCommandOpenApplication))
		{
		ButtonGroupContainer().SetCommandL(0, ENewCommandBrowse, KSelectLabel);
		// Only draw the one button to reduce flicker.
		ButtonGroupContainer().ButtonById(ENewCommandBrowse)->DrawNow();
		}
	// Note that we don't change the button when the action list is
	// selected, as this button is set automatically to "Change".
	else if (aControlId != ENewCommandCommandChoiceList)
		{
		ButtonGroupContainer().SetCommandL(0, ENewCommandBlank, KBlankLabel);
		ButtonGroupContainer().ButtonById(ENewCommandBlank)->DrawNow();
		}
	}

// This function starts recording a new command and sets up the member
// variable containing information about it.
void CNewCommandDialog::RecordL()
	{
	// Don't allow the user to enter two commands with the same name, or a
	// command without any name.
	iVoiceCommand->iName = ((CEikEdwin*) Control(ENewCommandNameEditor))->GetTextInHBufL();
	if (iVoiceCommand->iName == NULL)
		{
		_LIT(KEnterCommandName, "Enter a name for the command");
		iEikonEnv->InfoMsg(KEnterCommandName);
		return;
		}

	for (TInt i = 0; i < iParent.VoiceRecorder()->Commands().Count(); i++)
		{
		if (!iParent.VoiceRecorder()->Commands()[i]->iName->CompareF(*(iVoiceCommand->iName)))
			{
			_LIT(KCommandExists, "Command already exists");
			iEikonEnv->InfoMsg(KCommandExists);
			return;
			}
		}

	// Tell the user to wait until we're set up to start recording.
	_LIT(KPleaseWait, "Please wait");
	iEikonEnv->BusyMsgL(KPleaseWait, TGulAlignment(EHCenterVCenter));
	DrawNow();

	// Set up the command's member variables.
	iVoiceCommand->iCommand = (TVoiceCommandId) ((CEikChoiceList*) Control(ENewCommandCommandChoiceList))->CurrentItem();
	iVoiceCommand->iArgument = ((CEikEdwin*) Control(ENewCommandArgumentEditor))->GetTextInHBufL();
	if (iVoiceCommand->iArgument == NULL)
		iVoiceCommand->iArgument = HBufC::NewL(0);

	// Add the new command to the list of commands.
	iParent.AddCommandToListL(iVoiceCommand); // TAKES OWNERSHIP

	// Actually create the audio file for this voice command. This is
	// an asynchronous function.
	iParent.VoiceRecorder()->CreateVoiceFileL(KRawVoiceFile(), *this);

	// Set up the CBA with some new commands.
	ButtonGroupContainer().SetCommandL(0, ENewCommandBlank, KBlankLabel);
	ButtonGroupContainer().SetCommandL(1, ENewCommandBlank, KBlankLabel);
	ButtonGroupContainer().SetCommandL(2, ENewCommandBlank, KBlankLabel);
	ButtonGroupContainer().SetCommandL(3, ENewCommandStopRecording, KDoneLabel);

	ButtonGroupContainer().SetDefaultCommand(ENewCommandStopRecording);

	// DrawNow panics with user 20!
	ButtonGroupContainer().DrawDeferred();
	}

// This function starts re-recording an existing command and sets up the member
// variable containing information about it.
void CNewCommandDialog::RerecordL()
	{
	// Don't allow the user to enter a command without any name.
	iVoiceCommand->iName = ((CEikEdwin*) Control(ENewCommandNameEditor))->GetTextInHBufL();
	if (iVoiceCommand->iName == NULL)
		{
		_LIT(KEnterCommandName, "Enter a name for the command");
		iEikonEnv->InfoMsg(KEnterCommandName);
		return;
		}

	// Tell the user to wait until we're set up to start recording.
	_LIT(KPleaseWait, "Please wait");
	iEikonEnv->BusyMsgL(KPleaseWait, TGulAlignment(EHCenterVCenter));
	DrawNow();
	
	// Change the currently selected command's values.
	ModifyCommandInListL();

	// Actually create the audio file for this voice command. This is
	// an asynchronous function.
	iParent.VoiceRecorder()->CreateVoiceFileL(KRawVoiceFile(), *this);

	// Set up the CBA with some new commands.
	ButtonGroupContainer().SetCommandL(0, ENewCommandBlank, KBlankLabel);
	ButtonGroupContainer().SetCommandL(1, ENewCommandBlank, KBlankLabel);
	ButtonGroupContainer().SetCommandL(2, ENewCommandBlank, KBlankLabel);
	ButtonGroupContainer().SetCommandL(3, ENewCommandStopRecording, KDoneLabel);

	ButtonGroupContainer().SetDefaultCommand(ENewCommandStopRecording);

	// DrawNow panics with user 20!
	ButtonGroupContainer().DrawDeferred();
	}

void CNewCommandDialog::StopRecordingL()
	{
	// Actually stop recording, and show the user that we've done so.
	iParent.VoiceRecorder()->Stop();
	iEikonEnv->BusyMsgCancel();

	// Attempt to convert the voice file into a more sensible format.
	// This will return an error if there isn't any sound in the file.
	TInt rc = ConvertVoiceFileL();
	if (rc == KErrNotFound)
		return;

	// Increase the number of times this command has been recorded. If
	// we've recorded it enough times, we can stop.
	iNumRecorded++;

	if (iNumRecorded >= iParent.VoiceRecorder()->RecordRepetitions())
		{
		// Close the dialog when we've finished.
		TryExitL(ENewCommandStopRecording);
		}
	else
		{
		// Let the user know (modally) that we've still got a way to go.
		TBuf<256> alertText;
		_LIT(KAlertFormat, "Recorded command %d of %d times.");
		_LIT(KAlertRepeat, "Please repeat it.");
		_LIT(KPleaseWait, "Please wait");
		alertText.Format(KAlertFormat, iNumRecorded,
			iParent.VoiceRecorder()->RecordRepetitions());
		iEikonEnv->AlertWin(alertText, KAlertRepeat);
		iEikonEnv->BusyMsgL(KPleaseWait, TGulAlignment(EHCenterVCenter));
		DrawNow();

		// Start the recording process again.
		iParent.VoiceRecorder()->CreateVoiceFileL(KRawVoiceFile(), *this);
		}
	}

// This function will take a file containing audio data, strip any
// leading or trailing silence, and then generate information about it.
// This information will be stored with the current voice command.
TInt CNewCommandDialog::ConvertVoiceFileL()
	{
	// Remove silence from the file.
	TInt rc = iParent.VoiceRecorder()->CompactVoiceFileL(KCompactVoiceFile(),
		KRawVoiceFile());
	
	// If the user didn't actually record anything, try again.
	if (rc == KErrNotFound)
		{
		_LIT(KNoSpeechRecorded, "No speech was recorded."); 
		_LIT(KTryAgain, "Please try again.");
		iEikonEnv->AlertWin(KNoSpeechRecorded, KTryAgain);
		iParent.VoiceRecorder()->CreateVoiceFileL(KRawVoiceFile(), *this);
		return rc;
		}

	// Create the information about the voice command that's stored with it.
	iParent.VoiceRecorder()->CreateSignatureL(KCompactVoiceFile(), iVoiceCommand,
		iNumRecorded);
	return KErrNone;
	}

// This function modifies the currently selected command in the parent
// dialog's list box so that it corresponds with the values the user has
// just entered. It's used when editing an already existing command.
void CNewCommandDialog::ModifyCommandInListL()
	{
	CVoiceCommand* command = iParent.GetSelectedCommand();

	// Overwrite the command's name.
	delete command->iName;
	command->iName = NULL;
	command->iName = ((CEikEdwin*) Control(ENewCommandNameEditor))->GetTextInHBufL();

	// Overwrite the action to be performed.
	command->iCommand = (TVoiceCommandId) ((CEikChoiceList*) Control(ENewCommandCommandChoiceList))->CurrentItem();

	// Overwrite the argument.
	delete command->iArgument;
	command->iArgument = NULL;
	command->iArgument = ((CEikEdwin*) Control(ENewCommandArgumentEditor))->GetTextInHBufL();
	if (iVoiceCommand->iArgument == NULL)
		iVoiceCommand->iArgument = HBufC::NewL(0);

	// Update the list box.
	iParent.UpdateCommandList();
	}

// This function will be called when the audio recorder utility's state
// changes. We use it to determine whether we're ready to start recording.
void CNewCommandDialog::MoscoStateChangeEvent(CBase* /*aObject*/, TInt /*aPreviousState*/, TInt aCurrentState, TInt /*aErrorCode*/)
	{
	iState = aCurrentState;

	if (iState == KReadyToRecord)
		{
		iParent.VoiceRecorder()->RecordL();
		_LIT(KRecording, "Recording");
		iEikonEnv->BusyMsgL(KRecording, TGulAlignment(EHCenterVCenter));
		}
	}


//------------------------------------------------
// Definition of the "Select application" dialog.
//------------------------------------------------

// Standard C++ constructor. Takes a reference to a descriptor which will
// be filled in with the path name of the application chosen.
CSelectApplicationDialog::CSelectApplicationDialog(TDes& aApplicationPath)
	: iSelectedApplicationPath(aApplicationPath)
	{
	}

// Frees memory allocated.
CSelectApplicationDialog::~CSelectApplicationDialog()
	{
	delete iApplicationPaths;
	}

void CSelectApplicationDialog::PreLayoutDynInitL()
	{
	// Create the array of application paths, specifying an arbitrary
	// granularity.
	iApplicationPaths = new (ELeave) CDesCArrayFlat(8);

	iListBox = (CEikTextListBox*) Control(EVoiceControlApplicationList);

	CEikScrollBarFrame* scrollbarFrame = iListBox->CreateScrollBarFrameL();
	// Set horizontal and vertical scroll bars to appear when required.
	scrollbarFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
		CEikScrollBarFrame::EAuto);

	// Open the app architecture session for listing apps.
	RApaLsSession lsSession;
	User::LeaveIfError(lsSession.Connect());
	CleanupClosePushL(lsSession);

	TApaAppInfo info;
	// Start the process of getting application info.
	lsSession.GetAllApps();

	// While there are still applications to get, append their captions to
	// the list box and their paths to the array of paths.
	while (lsSession.GetNextApp(info) == KErrNone)
		{
		((CDesCArray*)iListBox->Model()->ItemTextArray())->AppendL(info.iCaption);
		iApplicationPaths->AppendL(info.iFullName);
		}
	iListBox->HandleItemAdditionL();
	CleanupStack::PopAndDestroy(); // lsSession
	}

// This function is called when the user closes the dialog.
TBool CSelectApplicationDialog::OkToExitL(TInt /*aButtonId*/)
	{
	// If the user selected an application, update the original caller's
	// app path.
	if (iListBox->CurrentItemIndex() >= 0)
		iSelectedApplicationPath.Copy((*iApplicationPaths)[iListBox->CurrentItemIndex()]);
	return ETrue;
	}
