#include "VoiceControlDialog.h"
#include "NewCommandDialog.h"

////////////////////////////////////////////////////////////////
//
// EXPORTed functions
//
////////////////////////////////////////////////////////////////

// The main DLL entry point.
GLDEF_C TInt E32Dll(TDllReason)
	{
	return KErrNone;
	}

// The function to create the main dialog. It is passed in the full
// path of the DLL used, which can be used to find the associated
// resource file.
EXPORT_C void CreateControlL(const TDesC& aDllFullPathName)
	{
	// Find the dialog's resource file from the path name we were passed.
	CCoeEnv* coeEnv = CCoeEnv::Static();
	TParse* parse = new(ELeave) TParse;
	CleanupStack::PushL(parse);
	_LIT(KRscSuffix, ".RSC");
	User::LeaveIfError(parse->SetNoWild(KRscSuffix, &aDllFullPathName, NULL));
	TFileName* resourceFileName=new (ELeave) TFileName(parse->FullName());
	CleanupStack::PushL(resourceFileName);
	BaflUtils::NearestLanguageFile(coeEnv->FsSession(), *resourceFileName);
	const TInt resourceFileId = coeEnv->AddResourceFileL(*resourceFileName);

	// Create the dialog.
	CVoiceControlDialog* dialog = new CVoiceControlDialog(resourceFileId); // does not leave
	if (dialog == NULL)
		{
		coeEnv->DeleteResourceFile(resourceFileId);
		User::LeaveNoMemory();
		}

	// We need to set the default path to be used for future file access, because
	// it is always initialised to c: and the voice control files may be stored
	// on the d: drive.
	TFileName fileName;
	fileName.Copy(parse->Drive());
	dialog->SetDriveLetter(parse->Drive());
	_LIT(KBackslash, "\\");
	fileName.Append(KBackslash);
	coeEnv->FsSession().SetSessionPath(fileName);

	CleanupStack::PopAndDestroy(2); // parse, resourceFileName

	// Display the dialog and then destroy it.
	dialog->ExecuteLD(R_VOICE_CONTROL_DIALOG);
	}

////////////////////////////////////////////////////////////////
//
// Main dialog class, CVoiceControlDialog
//
////////////////////////////////////////////////////////////////

CVoiceControlDialog::CVoiceControlDialog(TInt aResourceFileId)
	: CEikDialog(), iResourceFileId(aResourceFileId)
	{
	}

CVoiceControlDialog::~CVoiceControlDialog()
	{
	// Delete temporary files to free up disk space.
	iEikonEnv->FsSession().Delete(KRawVoiceFile());
	iEikonEnv->FsSession().Delete(KCompactVoiceFile());
	delete iVoiceRecorder;
	}

void CVoiceControlDialog::PreLayoutDynInitL()
	{
	// Create the voice recorder object and load its settings.
	iVoiceRecorder = new (ELeave) CVoiceRecorder(iEikonEnv->FsSession());
	iVoiceRecorder->InternalizeL();

	// Kill the voice daemon so it doesn't interfere with us.
	TApaTaskList taskList(iEikonEnv->WsSession());
	TUid daemonUid;
	daemonUid.iUid = KUidVoiceDaemon.iUid;
	TApaTask daemon = taskList.FindApp(daemonUid);
	daemon.EndTask();

	iListBox = (CEikTextListBox*) Control(EVoiceControlCommandList);

	CEikScrollBarFrame* scrollbarFrame = iListBox->CreateScrollBarFrameL();
	// Set horizontal and vertical scroll bars to appear when required.
	scrollbarFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
		CEikScrollBarFrame::EAuto);

	// Get the names of the voice commands into the list of commands.
	CDesCArray* listBoxArray = ((CDesCArray*)iListBox->Model()->ItemTextArray());
	iVoiceRecorder->GetVoiceCommandNamesL(*listBoxArray);

	iListBox->HandleItemAdditionL();

	// Set the initial values of some of the controls.
	iEnableVoiceControl = (CEikChoiceList*) Control(EVoiceControlEnable);
	if (iVoiceRecorder->EnableVoiceControl())
		iEnableVoiceControl->SetCurrentItem(0);
	else
		iEnableVoiceControl->SetCurrentItem(1);

	iEnablePlayTone = (CEikChoiceList*) Control(EVoiceControlPlayTone);
	if (iVoiceRecorder->EnablePlayTone())
		iEnablePlayTone->SetCurrentItem(0);
	else
		iEnablePlayTone->SetCurrentItem(1);

	CEikChoiceList* action = (CEikChoiceList*)
		Control(EVoiceControlAfterDialling);
	action->SetCurrentItem(iVoiceRecorder->ActionAfterDialling());

	// Find out which hotkey we've set to invoke voice control, and
	// update the text box containing the hotkey.
	iHotKeyEditor = (CEikEdwin*) Control(EVoiceControlHotKey);
	iHotKeyEditor->SetReadOnly(ETrue);
	TInt hotKeyCode;
	TUint hotKeyModifiers;
	iVoiceRecorder->GetHotKey(hotKeyCode, hotKeyModifiers);
	SetHotKeyL(hotKeyCode, hotKeyModifiers);

	// Set the current number of record repetitions.
	iRepetitionsList = (CEikChoiceList*) Control(EVoiceControlRepetitions);
	iRepetitionsList->SetCurrentItem(iVoiceRecorder->RecordRepetitions() - 1);

	// Update the CBA with the available actions.
	ButtonGroupContainer().SetCommandL(0, EVoiceControlNewCommand, KNewCommandLabel);
	ButtonGroupContainer().SetCommandL(1, EVoiceControlEditCommand, KEditCommandLabel);
	ButtonGroupContainer().SetCommandL(2, EVoiceControlRemoveCommand, KRemoveCommandLabel);
	ButtonGroupContainer().SetCommandL(3, EVoiceControlClose, KCloseLabel);
	ButtonGroupContainer().SetDefaultCommand(EVoiceControlClose);

	// Closing the voice daemon sometimes leads to this app being sent
	// to the back. We therefore need to place ourselves at the front again.
	TApaTask task(iEikonEnv->WsSession());
	task.SetWgId(iEikonEnv->RootWin().Identifier());
	task.BringToForeground();
	}

// This function will be called when the user tries to close the control
// panel applet. We always allow this.
TBool CVoiceControlDialog::OkToExitL(TInt /*aButtonId*/)
	{
	// Save the settings.
	iVoiceRecorder->ExternalizeL();

	// Restart the voice daemon if we need to.
	if (iVoiceRecorder->EnableVoiceControl())
		{
		// Create the path to the voice daemon and run it.
		RApaLsSession lsSession;
		User::LeaveIfError(lsSession.Connect());
		CleanupClosePushL(lsSession);

		TFileName voiceDaemonPath = iDriveLetter;
		voiceDaemonPath.Append(KVoiceDaemonPath());

		CApaCommandLine* cmdLine = CApaCommandLine::NewLC(voiceDaemonPath);
		lsSession.StartApp(*cmdLine);

		CleanupStack::PopAndDestroy(2); // lsSession, cmdLine
		}

	// Return that we're allowing the user to exit.
	return ETrue;
	}

// Setter function to set the drive letter this control panel extension's on.
void CVoiceControlDialog::SetDriveLetter(const TDesC& aDriveLetter)
	{
	iDriveLetter = aDriveLetter;
	}

// Getter function to return the drive the control panel extension's on.
const TDesC& CVoiceControlDialog::DriveLetter()
	{
	return iDriveLetter;
	}

// This function will be called on startup, and when the user changes
// the hotkey to invoke voice control. It updates the hotkey value
// stored in the voice recorder object, and updates the text box showing
// the hotkey.
void CVoiceControlDialog::SetHotKeyL(TUint aScanCode, TUint aModifiers)
	{
	// We're only interested in certain keys.
	const TUint modifierMask = EModifierAlt | EModifierCtrl |
		EModifierShift | EModifierFunc | EModifierCapsLock;
	
	aModifiers &= modifierMask;

	iVoiceRecorder->SetHotKey(aScanCode, aModifiers);

	// Generate a text string from the hotkey.
	TBuf<256> hotKeyLine;
	hotKeyLine.Zero();

	_LIT(KModAlt, "Alt + ");
	_LIT(KModCtrl, "Ctrl + ");
	_LIT(KModShift, "Shift + ");
	_LIT(KModChr, "Chr + ");
	_LIT(KModCaps, "Caps + ");

	_LIT(KBackspace, "Backspace");
	_LIT(KTab, "Tab");
	_LIT(KEnter, "Enter");
	_LIT(KEscape, "Escape");
	_LIT(KSpace, "Space");
	_LIT(KLeftArrow, "Left arrow");
	_LIT(KRightArrow, "Right arrow");
	_LIT(KUpArrow, "Up arrow");
	_LIT(KDownArrow, "Down arrow");
	_LIT(KDelete, "Delete");
	_LIT(KComma, "Comma");
	_LIT(KFullStop, "Full stop");
	_LIT(KSlash, "/");
	_LIT(KMenu, "Menu");
	_LIT(KUnknown, "Unknown");

	if (aModifiers & EModifierAlt)
		hotKeyLine.Append(KModAlt);
	if (aModifiers & EModifierCtrl)
		hotKeyLine.Append(KModCtrl);
	if (aModifiers & EModifierShift)
		hotKeyLine.Append(KModShift);
	if (aModifiers & EModifierFunc)
		hotKeyLine.Append(KModChr);
	if (aModifiers & EModifierCapsLock)
		hotKeyLine.Append(KModCaps);
	
	// If it's a standard ASCII key, just display it. Otherwise,
	// choose what to display based on its scancode.
	if ((aScanCode > EStdKeyScrollLock) && (aScanCode < EStdKeyF1))
		{
		hotKeyLine.Append(TChar(aScanCode));
		}
	else
		{
		if (aScanCode == EStdKeyBackspace)
			hotKeyLine.Append(KBackspace);
		else if (aScanCode == EStdKeyTab)
			hotKeyLine.Append(KTab);
		else if (aScanCode == EStdKeyEnter)
			hotKeyLine.Append(KEnter);
		else if (aScanCode == EStdKeyEscape)
			hotKeyLine.Append(KEscape);
		else if (aScanCode == EStdKeySpace)
			hotKeyLine.Append(KSpace);
		else if (aScanCode == EStdKeyLeftArrow)
			hotKeyLine.Append(KLeftArrow);
		else if (aScanCode == EStdKeyRightArrow)
			hotKeyLine.Append(KRightArrow);
		else if (aScanCode == EStdKeyUpArrow)
			hotKeyLine.Append(KUpArrow);
		else if (aScanCode == EStdKeyDownArrow)
			hotKeyLine.Append(KDownArrow);
		else if (aScanCode == EStdKeyDelete)
			hotKeyLine.Append(KDelete);
		else if (aScanCode == EStdKeyComma)
			hotKeyLine.Append(KComma);
		else if (aScanCode == EStdKeyFullStop)
			hotKeyLine.Append(KFullStop);
		else if (aScanCode == EStdKeyForwardSlash)
			hotKeyLine.Append(KSlash);
		else if (aScanCode == EStdKeyMenu)
			hotKeyLine.Append(KMenu);
		else
			hotKeyLine.Append(KUnknown);
		}
	// Update the text box containing the hotkey.
	iHotKeyEditor->SetTextL(&hotKeyLine);
	iHotKeyEditor->SetSelectionL(0, 0);
	iHotKeyEditor->DrawNow();
	}

// This function will be called when any key is pressed (including the
// four CBA keys).
TKeyResponse CVoiceControlDialog::OfferKeyEventL(const TKeyEvent& aKeyEvent,
												 TEventCode aType)
	{
	if (aType == EEventKey)
		{
		switch (aKeyEvent.iScanCode)
			{
			// If the top CBA button is pressed, create a new command.
		case EStdKeyDevice0:
			NewCommandL();
			break;

			// Edit an existing command when this button is pressed.
		case EStdKeyDevice1:
			EditCommandL();
			break;

			// Remove a command when the third button is pressed. The Nokia
			// style guide stipulates that destructive actions should only
			// be performed by this button.
		case EStdKeyDevice2:
			RemoveCommandL();
			break;

			// Attempt to exit when the bottom button is pressed.
		case EStdKeyDevice3:
			TryExitL(3);
			break;

		default:
			// If the hotkey editor control is focused, we try to update it.
			// Only set the hotkey when the key has a modifier set.
			// We have to use scan codes because, when the ctrl / func
			// keys are pressed, key codes are dodgy (eg. "a" is 1).
			if (iHotKeyEditor->IsFocused() &&
				(aKeyEvent.iModifiers > EModifierAutorepeatable))
				{
				SetHotKeyL(aKeyEvent.iScanCode, aKeyEvent.iModifiers);
				}
			else
				{
				CEikDialog::OfferKeyEventL(aKeyEvent, aType);
				}
			break;
			}
		}
	// We have to return this, or the event gets passed to the control panel!
	return EKeyWasConsumed;
	}

// This function is called when a control experiences any event.
// We use it to update the internal settings. Note that we don't use
// it for the hotkey editor - this relies on trapping keypresses.
void CVoiceControlDialog::HandleControlEventL(CCoeControl* aControl, TCoeEvent /*aEventType*/)
	{
	if (aControl == iRepetitionsList)
		{
		iVoiceRecorder->SetRecordRepetitions(iRepetitionsList->CurrentItem() + 1);
		}
	else if (aControl == Control(EVoiceControlAfterDialling))
		{
		CEikChoiceList* choiceList = (CEikChoiceList*) Control(EVoiceControlAfterDialling);
		iVoiceRecorder->SetActionAfterDialling((TActionAfterDialling)
			choiceList->CurrentItem());
		}
	else if (aControl == iEnableVoiceControl)
		{
		if (iEnableVoiceControl->CurrentItem() == 0)
			iVoiceRecorder->SetEnableVoiceControl(ETrue);
		else
			iVoiceRecorder->SetEnableVoiceControl(EFalse);
		}
	else if (aControl == iEnablePlayTone)
		{
		if (iEnablePlayTone->CurrentItem() == 0)
			iVoiceRecorder->SetEnablePlayTone(ETrue);
		else
			iVoiceRecorder->SetEnablePlayTone(EFalse);
		}
	}

// This function will be called when the user changes from the "Commands"
// page to the "Settings" page, or vice versa. It updates the CBA based
// on the new current page.
void CVoiceControlDialog::PageChangedL(TInt aPageId)
	{
	if (aPageId == EVoiceControlCommands)
		{
		ButtonGroupContainer().SetCommandL(0, EVoiceControlNewCommand, KNewCommandLabel);
		ButtonGroupContainer().SetCommandL(1, EVoiceControlEditCommand, KEditCommandLabel);
		ButtonGroupContainer().SetCommandL(2, EVoiceControlRemoveCommand, KRemoveCommandLabel);
		}
	else
		{
		ButtonGroupContainer().SetCommandL(0, EVoiceControlBlank, KBlankLabel);
		ButtonGroupContainer().SetCommandL(1, EVoiceControlBlank, KBlankLabel);
		ButtonGroupContainer().SetCommandL(2, EVoiceControlBlank, KBlankLabel);
		}
	ButtonGroupContainer().DrawNow();
	}

void CVoiceControlDialog::NewCommandL()
	{
	// Create the "new command" dialog.
	CEikDialog* dialog = new CNewCommandDialog(*this, iResourceFileId,
		EFalse); // does not leave
	// Leave if we couldn't create it.
	if (dialog == NULL)
		User::LeaveNoMemory();
	// Execute the dialog modally. The "D" suffix means that it will be
	// deleted for us.
	dialog->ExecuteLD(R_NEW_COMMAND_DIALOG);
	}

void CVoiceControlDialog::EditCommandL()
	{
	// Check that we're not trying to edit a nonexistent command.
	if (iListBox->Model()->NumberOfItems() > 0)
		{
		// Create the "edit command" dialog.
		CEikDialog* dialog = new CNewCommandDialog(*this, iResourceFileId,
			ETrue); // does not leave
		// Leave if we couldn't create it.
		if (dialog == NULL)
			User::LeaveNoMemory();
		// Execute the dialog modally. The "D" suffix means that it will be
		// deleted for us.
		dialog->ExecuteLD(R_NEW_COMMAND_DIALOG);
		}
	}

void CVoiceControlDialog::RemoveCommandL()
	{
	// Check that we're not trying to remove a nonexistent command.
	if (iListBox->Model()->NumberOfItems() > 0)
		{
		// Free the memory associated with the command and remove it
		// from the array of commands.
		delete iVoiceRecorder->Commands()[iListBox->CurrentItemIndex()];
		iVoiceRecorder->Commands().Remove(iListBox->CurrentItemIndex());

		UpdateCommandList();
		}
	}

void CVoiceControlDialog::AddCommandToListL(const CVoiceCommand* aCommand)
	{
	// Append the new command's name to the list box.
	CDesCArray* listBoxArray=((CDesCArray*)iListBox->Model()->ItemTextArray());
	listBoxArray->AppendL(*(aCommand->iName));
	iListBox->HandleItemAdditionL();

	// Append the new command to the list of commands.
	// Do this after the append, which can leave.
	iVoiceRecorder->Commands().Append(aCommand);
	}

// This function is called when the user presses the "Help" key. It
// should return the current help context (ie. what help to provide),
// and the UID of the help file to load from \System\Help.
void CVoiceControlDialog::GetHelpContext(TCoeHelpContext& aContext) const
	{
	aContext.iContext = KVoice_Control_Help;
	aContext.iMajor = KUidVoiceControl;
	}

// Simple "getter" function which returns a pointer to the voice
// recorder object.
CVoiceRecorder* CVoiceControlDialog::VoiceRecorder()
	{
	return iVoiceRecorder;
	}

CVoiceCommand* CVoiceControlDialog::GetSelectedCommand()
	{
	return iVoiceRecorder->Commands()[iListBox->CurrentItemIndex()];
	}

void CVoiceControlDialog::UpdateCommandList()
	{
	// Create a new array for the command names.
	CDesCArrayFlat* newArray = new (ELeave) CDesCArrayFlat(5);
	CleanupStack::PushL(newArray);
	
	// Get the voice command names into a descriptor array.
	iVoiceRecorder->GetVoiceCommandNamesL(*newArray);
	// Set the list box to contain the list of command names.
	iListBox->Model()->SetItemTextArray(newArray);
	// Make the list box responsible for freeing this list.
	iListBox->Model()->SetOwnershipType(ELbmOwnsItemArray);
	// Leave the list box in a sane state.
	iListBox->HandleItemRemovalL();

	// Don't destroy the array, because the list box owns it now.
	CleanupStack::Pop();

	iListBox->DrawNow();
	}