// This file defines the CVoiceDaemonDialer class, which provides
// asynchronous phone number dialling services.

#include <eikenv.h>
#include <eikbtgpc.h>

#include <apgtask.h>
#include <apgcli.h>

#include "VoiceDaemonDialer.h"
#include "VoiceDaemon.hrh"

// Simple constructor.
CVoiceDaemonDialer::CVoiceDaemonDialer(CVoiceDaemonDocument* aDocument)
	: CActive(EPriorityNormal), iDocument(aDocument)
	{
	CActiveScheduler::Add(this);
	}

// We don't need to destroy any member variables - calls etc. will already
// have been closed.
CVoiceDaemonDialer::~CVoiceDaemonDialer()
	{
	}

// This function starts the dialling process with the specified phone number.
void CVoiceDaemonDialer::StartL(const TDesC& aPhoneNumber)
	{
	// If the phone number dialled has spaces in it, the dialling request
	// never completes. We therefore remove all spaces before dialling.
	TBuf<256> numberWithoutSpaces;
	for (TInt i = 0; i < aPhoneNumber.Length(); i++)
		{
		// If there's an alphabetic character in the phone number,
		// stop copying now, as we won't understand anything after this point.
		if (TChar(aPhoneNumber[i]).IsAlpha())
			break;
		if (!TChar(aPhoneNumber[i]).IsSpace())
			numberWithoutSpaces.Append(aPhoneNumber[i]);
		}

	iPhoneNumber = numberWithoutSpaces;

	TBuf<256> formatBuf;
	_LIT(KDiallingFormat, "Dialling %S...");
	formatBuf.Format(KDiallingFormat, &numberWithoutSpaces);
	iDocument->SetDisplayText(formatBuf);

	// The name of the TSY to use to make the call.
	_LIT(KTsyName, "Phonetsy.tsy");
	RTelServer::TPhoneInfo info;
	RPhone::TLineInfo lineInfo;

	// Connect to the telephony server and load the TSY.
	User::LeaveIfError(iTelServer.Connect());
	User::LeaveIfError(iTelServer.LoadPhoneModule(KTsyName));

	// Get the details for the first (and only) phone.
	User::LeaveIfError(iTelServer.GetPhoneInfo(0, info));

	// Open the phone.
	User::LeaveIfError(iPhone.Open(iTelServer, info.iName));

	// Get the information for the voice line, line 0.
	User::LeaveIfError(iPhone.GetLineInfo(0, lineInfo));
	
	// Open the line. iName will now be "VoiceLine1".
	User::LeaveIfError(iPhoneLine.Open(iPhone, lineInfo.iName));

	// Open a new phone call.
	User::LeaveIfError(iPhoneCall.OpenNewCall(iPhoneLine));
	
	iDocument->SetCalling(ETrue);

	// Actually dial the number required.
	iPhoneCall.Dial(iStatus, numberWithoutSpaces);

	// Enable the "Hang up" button.
	CEikButtonGroupContainer* cba = CEikonEnv::Static()->AppUiFactory()->ToolBar();
	cba->SetCommandL(3, ECbaButtonHangUp, KHangUpLabel);
	cba->SetDefaultCommand(ECbaButtonHangUp);
	cba->DrawNow();

	SetActive();
	}

// This function will be called when the asynchronous dialling process
// completes, or there's been an error.
void CVoiceDaemonDialer::RunL()
	{
	RCall::TStatus callStatus;
	iPhoneCall.GetStatus(callStatus);
	TBuf<256> formatBuf;

	// If the user has cancelled the call, close the call and go to the back.
	// This occurs when the user presses the "Hang up" button while the phone
	// is still ringing.
	if (iStatus.Int() == KErrCancel)
		{
		CloseCall();
		iDocument->SendToBack();
		return;
		}

	// If there's been an error, hang up.
	if (iStatus.Int() != KErrNone)
		{
		HangUp();
		return;
		}

	// If we don't want to use our internal interface, we can close the
	// call object, becase we won't need it again.
	if (iDocument->VoiceRecorder()->ActionAfterDialling() !=
		EActionUseInternalUi)
		{
		CloseCall();
		iDocument->SendToBack();

		// Bring the phone application to the front if the user wants to.
		// Note that the reason we only do this after dialling has completed
		// is that the Telephone app doesn't update its display until this
		// has occurred. This can lead to the user thinking that the 9210
		// has locked up.
		if (iDocument->VoiceRecorder()->ActionAfterDialling() ==
			EActionBringPhoneToFront)
			{
			TApaTaskList taskList(CEikonEnv::Static()->WsSession());
			_LIT(KTelephoneApp, "Telephone");
			TApaTask phoneApp = taskList.FindApp(KTelephoneApp);
			phoneApp.BringToForeground();
			}

		return;
		}

	_LIT(KConnectedFormat, "Connected to %S");
	_LIT(KRingingFormat, "%S is ringing");
	_LIT(KAnsweringFormat, "%S is answering");
	_LIT(KEngagedFormat, "%S is engaged");
	// Update the display based on what's going on.
	switch (callStatus)
		{
	case RCall::EStatusConnected:
		formatBuf.Format(KConnectedFormat, &iPhoneNumber);
		iDocument->SetDisplayText(formatBuf);
		break;

	case RCall::EStatusRinging:
		formatBuf.Format(KRingingFormat, &iPhoneNumber);
		iDocument->SetDisplayText(formatBuf);
		break;

	case RCall::EStatusAnswering:
		formatBuf.Format(KAnsweringFormat, &iPhoneNumber);
		iDocument->SetDisplayText(formatBuf);
		break;

	case RCall::EStatusUnknown:
		formatBuf.Format(KEngagedFormat, &iPhoneNumber);
		iDocument->SetDisplayText(formatBuf);
		break;

	case RCall::EStatusIdle:
		// If the call's complete, close it and hide the app.
		CloseCall();
		iDocument->SendToBack();
		break;

	default:
		break;
		}
	}

// This function hangs up the call.
void CVoiceDaemonDialer::HangUp()
	{
	RCall::TStatus callStatus;
	iPhoneCall.GetStatus(callStatus);

	switch (callStatus)
		{
	case RCall::EStatusDialling:
		// Cancel dialling.
		iPhoneCall.DialCancel();
		break;

	case RCall::EStatusConnected:
	case RCall::EStatusRinging:
	case RCall::EStatusAnswering:
	case RCall::EStatusConnecting:
		// Hang up, and mark this object as active so it gets rescheduled.
		iPhoneCall.HangUp(iStatus);
		SetActive();
		break;

		// We get EStatusUnknown when the remote end's engaged, or still
		// ringing, or a mobile phone's voicemail.
	case RCall::EStatusUnknown:
		iPhoneCall.HangUp(iStatus);
		SetActive();
		break;

	case RCall::EStatusIdle:
	default:
		CloseCall();
		iDocument->SendToBack();
		break;
		}
	}

void CVoiceDaemonDialer::DoCancel()
	{
	CloseCall();
	}

void CVoiceDaemonDialer::CloseCall()
	{
	// Close all of the call-related objects.
	iPhoneCall.Close();
	iPhoneLine.Close();
	iPhone.Close();
	iTelServer.Close();
	iDocument->SetCalling(EFalse);

	// Reset the buttons now we're not on the phone any more.
	CEikButtonGroupContainer* cba = CEikonEnv::Static()->AppUiFactory()->ToolBar();
	cba->SetCommandL(3, ECbaButtonBlank, KBlankLabel);
	}