#include <cntdb.h>
#include <cntfield.h>
#include <cntitem.h>
#include <cntfldst.h>
#include <eikenv.h>
#include <eikbtgpc.h>
#include <apgtask.h>
#include <apgcli.h>

#include "VoiceDaemonDocument.h"
#include "VoiceDaemonDialer.h"
#include "VoiceDaemonAppUi.h"

////////////////////////////////////////////////////////////////
//
// Document class, CVoiceDaemonDocument
//
////////////////////////////////////////////////////////////////

// Frequency and duration of the tone to be played
const TInt KSoundFrequency = 2500;
const TInt KSoundDuration = 500000;

// C++ constructor
CVoiceDaemonDocument::CVoiceDaemonDocument(CEikApplication& aApp)
		: CEikDocument(aApp)
	{
	}

// Factory function for the CVoiceDocument class.
CVoiceDaemonDocument* CVoiceDaemonDocument::NewL(CEikApplication& aApp)
	{
	CVoiceDaemonDocument* self = new (ELeave) CVoiceDaemonDocument(aApp);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
	}

void CVoiceDaemonDocument::ConstructL()
	{
	// Get a pointer to the environment.
	iEikEnv = CEikonEnv::Static();

	// Send this application to the back ASAP.
	SendToBack();

	// Create the asynchronous dialer object.
	iDialer = new (ELeave) CVoiceDaemonDialer(this);

	// We need to set the default path to be used for future file access, because
	// it is always initialised to c: and the voice control files may be stored
	// on the d: drive.
	TParse parse;
	parse.SetNoWild(Application()->AppFullName(), NULL, NULL);

	TFileName fileName;
	fileName.Copy(parse.Drive());
	_LIT(KBackslash, "\\");
	fileName.Append(KBackslash);
	iEikEnv->FsSession().SetSessionPath(fileName);

	// Create the voice recorder object and load its settings.
	iVoiceRecorder = new (ELeave) CVoiceRecorder(iEikEnv->FsSession());
	iVoiceRecorder->InternalizeL();

	// Create the tone player object.
	iMdaAudioToneUtility = CMdaAudioToneUtility::NewL(*this);

	// Prepare to record our first command.
	PrepareForNextCommandL();
	}

// All resources allocated in ConstructL() must be released in 
// the destructor.
CVoiceDaemonDocument::~CVoiceDaemonDocument()
	{
	// Delete temporary files to free up disk space.
	iEikEnv->FsSession().Delete(KRawVoiceFile());
	iEikEnv->FsSession().Delete(KCompactVoiceFile());

	delete iDialer;
	delete iVoiceRecorder;
	delete iMdaAudioToneUtility;
	}

CEikAppUi* CVoiceDaemonDocument::CreateAppUiL()
	{
    return new (ELeave) CVoiceDaemonAppUi;
	}

CVoiceRecorder* CVoiceDaemonDocument::VoiceRecorder()
	{
	return iVoiceRecorder;
	}

CMdaAudioToneUtility* CVoiceDaemonDocument::MdaAudioToneUtility()
	{
	return iMdaAudioToneUtility;
	}

// This function will be called when the audio recorder utility's state
// changes - for example, when it's ready to record, or has finished
// recording.
void CVoiceDaemonDocument::MoscoStateChangeEvent(CBase* /*aObject*/, TInt /*aPreviousState*/, TInt aCurrentState, TInt /*aErrorCode*/)
	{
	iState = aCurrentState;
	
	if (iState == KReadyToRecord)
		{
		// Now we're ready to record, capture the hotkey
		((CVoiceDaemonAppUi*) iAppUi)->CaptureKey();
		}
	}

// This function will be called when the audio tone utility is
// ready to play the tone. Before actually playing the tone, we
// create the voice command file, because this process can take
// some time and we want to respond to the user quickly.
void CVoiceDaemonDocument::MatoPrepareComplete(TInt /*aError*/)
	{
	iMdaAudioToneUtility->SetVolume(iMdaAudioToneUtility->MaxVolume());

	// Ignore any error.
	TRAPD(err, iVoiceRecorder->CreateVoiceFileL(KRawVoiceFile(), *this));
	}

// This function will be called when the tone has finished playing.
// Once we get to this point, we want to actually start recording.
void CVoiceDaemonDocument::MatoPlayComplete(TInt /*aError*/)
	{
	// Ignore any error.
	TRAPD(err, iVoiceRecorder->RecordL());
	_LIT(KRecording, "Recording...");
	((CVoiceDaemonAppUi*) iAppUi)->SetDisplayText(KRecording);
	}

// This function uses the CVoiceDaemonDialer class to actually dial the
// specified phone number.
void CVoiceDaemonDocument::DialPhoneNumberL(const TDesC& aPhoneNumber)
	{
	// Don't allow the user to press any buttons while we're on the phone.
	// If the app is set to use its internal UI for hanging up, the "hang
	// up" button will be re-enabled later.
	CEikButtonGroupContainer* cba = iEikEnv->AppUiFactory()->ToolBar();
	cba->SetCommandL(0, ECbaButtonBlank, KBlankLabel);
	cba->SetCommandL(3, ECbaButtonBlank, KBlankLabel);
	cba->DrawNow();

	// Start the asynchronous dialer object with the specified phone number
	TRAPD(err, iDialer->StartL(aPhoneNumber));

	// In the event of an error, send the application to the back
	if (err != KErrNone)
		{
		User::Leave(err);
		SendToBack();
		}
	}

// This function splits a string (such as a name) into words. It assumes
// that any character which is non-alphanumeric is space and can hence be
// discarded. Its purpose is to enable the finding of contacts which are
// specified as a full name.
TInt CVoiceDaemonDocument::SplitContactWordsL(TAny* aPtr)
	{
	SFindInTextDefWordParser* parser = (SFindInTextDefWordParser*) aPtr;
	TInt wordStart = 0;
	TBool inWord = ETrue;
	TPtrC searchPtr(parser->iSearchString->Ptr(), parser->iSearchString->Length());

	for (TInt i = 0; i <= searchPtr.Length(); i++)
		{
		if (i == searchPtr.Length())
			{
			if (inWord)
				parser->iWordArray->AppendL(searchPtr.Mid(wordStart, i - wordStart));
			break;
			}
		if (TChar(searchPtr[i]).IsAlphaDigit())
			{
			if (!inWord)
				wordStart = i;
			inWord = ETrue;
			}
		else
			{
			if (inWord)
				parser->iWordArray->AppendL(searchPtr.Mid(wordStart, i - wordStart));
			inWord = EFalse;
			}
		}
	return KErrNone;
	}

// This function dials a contact by name. It can either dial their default
// phone number, or their default mobile phone number.
TInt CVoiceDaemonDocument::DialContactL(const TDesC& aContactName, TBool aMobile)
	{
	// Create a descriptor array with an arbitrary granularity. This will
	// be used to store the words that make up the contact name we were
	// passed.
	CDesCArrayFlat* wordArray = new (ELeave) CDesCArrayFlat(2);
	CleanupStack::PushL(wordArray);

	// Parse the contact name we were passed into separate words.
	SFindInTextDefWordParser parser(&aContactName, wordArray);
	SplitContactWordsL(&parser);

	// Open the contacts database.
	CContactDatabase* db = CContactDatabase::OpenL();
	CleanupStack::PushL(db);

	// Create a text field definition, using the family and
	// given name fields, as these are the ones we want to search in.
	TContactTextDefItem familyName(KUidContactFieldFamilyName);
	TContactTextDefItem givenName(KUidContactFieldGivenName);

	CContactTextDef* contactTextDef = CContactTextDef::NewL();
	CleanupStack::PushL(contactTextDef);

	contactTextDef->AppendL(givenName);
	contactTextDef->AppendL(familyName);

	// Actually attempt to find the specified contact.
	CContactIdArray* contacts = db->FindInTextDefLC(*wordArray, contactTextDef,
		TCallBack(SplitContactWordsL));

	if (contacts->Count() == 0)
		{
		CleanupStack::PopAndDestroy(4); // wordArray, db, contactTextDef, contacts
		return KErrNotFound;
		}
	
	// Get the details of the first contact we found.
	CContactItem* contact = db->ReadContactLC((*contacts)[0]);

	CContactItemFieldSet& fieldSet = contact->CardFields();

	// Find the index of the phone number field. If the "mobile" flag is
	// specified, we specifically try to find the mobile number. Otherwise,
	// we just use the standard phone number field. Note that this will return
	// the first mobile number if no land line number exists.
	TInt numFieldIndex;
	if (!aMobile)
		numFieldIndex = fieldSet.Find(KUidContactFieldPhoneNumber);
	else
		numFieldIndex = fieldSet.Find(KUidContactFieldVCardMapCELL);

	if (numFieldIndex < 0)
		User::Leave(KErrNotFound);

	// Extract the text of the phone number from the contact field.
	CContactItemField& numField = fieldSet[numFieldIndex];
	CContactTextField* numText = numField.TextStorage();

	// Dial the number we've extracted.
	DialPhoneNumberL(numText->Text());

	CleanupStack::PopAndDestroy(5); // db, wordArray, contactTextDef, contacts, contact

	return KErrNone;
	}

void CVoiceDaemonDocument::HangUpL()
	{
	iDialer->HangUp();
	}

void CVoiceDaemonDocument::SetDisplayText(const TDesC& aText)
	{
	((CVoiceDaemonAppUi*) iAppUi)->SetDisplayText(aText);
	}

// This function actually executes a voice command. These can perform
// various actions.
void CVoiceDaemonDocument::ExecuteCommandL(CVoiceCommand* aCommand)
	{
	_LIT(KContactErrorFormat, "Couldn't find contact \"%S\".");
	_LIT(KOpenErrorFormat, "Couldn't open \"%S\".");
	_LIT(KRunErrorFormat, "Couldn't run \"%S\".");
	_LIT(KCheckSettings, "Check your settings.");
	_LIT(KUnrecognisedCommand, "Unrecognised command");
	
	switch (aCommand->iCommand)
		{
	case EVoiceCommandDialContact:
		// Attempt to dial a contact's normal telephone number. If the
		// contact cannot be found, notify the user.
		if (DialContactL(*(aCommand->iArgument), EFalse) == KErrNotFound)
			{
			TBuf<256> errBuf;
			errBuf.Format(KContactErrorFormat, aCommand->iArgument);
			iEikEnv->AlertWin(errBuf);
			SendToBack();
			}
		break;

	case EVoiceCommandDialContactMobile:
		// Attempt to dial a contact's mobile telephone number. If the
		// contact can't be found, notify the user.
		if (DialContactL(*(aCommand->iArgument), ETrue) == KErrNotFound)
			{
			TBuf<256> errBuf;
			errBuf.Format(KContactErrorFormat, aCommand->iArgument);
			iEikEnv->AlertWin(errBuf);
			SendToBack();
			}
		break;

	case EVoiceCommandDialNumber:
		// Simply dial a phone number.
		DialPhoneNumberL(*(aCommand->iArgument));
		break;

	case EVoiceCommandOpenDocument:
		{
		// Go to the back now, before the document starts,
		// so that we're still at the back when it terminates.
		SendToBack();

		// Load the requested document.
		RApaLsSession lsSession;
		TThreadId appId;
		User::LeaveIfError(lsSession.Connect());
		CleanupClosePushL(lsSession);
		TInt rc = lsSession.StartDocument(*(aCommand->iArgument), appId);

		// If we couldn't open the document, notify the user.
		if (rc != KErrNone)
			{
			TBuf<256> pathBuf;
			pathBuf.Format(KOpenErrorFormat, aCommand->iArgument);
			BringToFront();
			iEikEnv->AlertWin(pathBuf, KCheckSettings);
			SendToBack();
			}

		CleanupStack::PopAndDestroy(); // lsSession
		}
		break;

	case EVoiceCommandOpenApplication:
		{
		// Go to the back now, before the application starts,
		// so that we're still at the back when it terminates.
		SendToBack();

		// Start the application with the specified command line.
		RApaLsSession lsSession;
		User::LeaveIfError(lsSession.Connect());
		CleanupClosePushL(lsSession);
		CApaCommandLine* cmdLine = CApaCommandLine::NewLC(*(aCommand->iArgument));
		TInt rc = lsSession.StartApp(*cmdLine);

		// Notify the user if we couldn't start the app.
		if (rc != KErrNone)
			{
			TBuf<256> pathBuf;
			pathBuf.Format(KRunErrorFormat, aCommand->iArgument);
			BringToFront();
			iEikEnv->AlertWin(pathBuf, KCheckSettings);
			SendToBack();
			}

		CleanupStack::PopAndDestroy(2); // lsSession, cmdLine
		}
		break;

	default:
		// This should never happen.
		((CVoiceDaemonAppUi*) iAppUi)->SetDisplayText(KUnrecognisedCommand);
		break;
		}
	}

void CVoiceDaemonDocument::BringToFront()
	{
	_LIT(KPleaseWait, "Please wait...");

	// Tell the user not to start recording a command just yet.
	if (iAppUi != NULL)
		((CVoiceDaemonAppUi*) iAppUi)->SetDisplayText(KPleaseWait);

	// Bring this application to the foreground.
	TApaTask task(iEikEnv->WsSession());
	task.SetWgId(iEikEnv->RootWin().Identifier());
	task.BringToForeground();
	}

void CVoiceDaemonDocument::SendToBack()
	{
	// Send this application to the background.
	TApaTask task(iEikEnv->WsSession());
	task.SetWgId(iEikEnv->RootWin().Identifier());
	task.SendToBackground();

	// This is another method of hiding the application.
//	iEikEnv->RootWin().SetOrdinalPosition(-1);

	// If the user changes focus to this app, we want to reassure them
	// that it's OK to speak a command.
	_LIT(KPressHotkey, "Press the hotkey to record");
	if (iAppUi != NULL)
		((CVoiceDaemonAppUi*) iAppUi)->SetDisplayText(KPressHotkey);
	}

// This function starts the long-winded process of preparing to record
// another command. This will be different depending on whether we need
// to beep before the user records the next command or not.
void CVoiceDaemonDocument::PrepareForNextCommandL()
	{
	if (iVoiceRecorder->EnablePlayTone())
		{
		// Prepare to play the beep.
		iMdaAudioToneUtility->PrepareToPlayTone(KSoundFrequency,
			TTimeIntervalMicroSeconds(KSoundDuration));
		}
	else
		{
		// Open the file into which the next command will be recorded.
		iVoiceRecorder->CreateVoiceFileL(KRawVoiceFile(), *this);
		}
	}

// Setter/getter functions related to whether a phone call is happening.

void CVoiceDaemonDocument::SetCalling(TBool aCalling)
	{
	iCalling = aCalling;
	}

TBool CVoiceDaemonDocument::Calling()
	{
	return iCalling;
	}