// This is the header file for the Voice Recorder DLL.
// It defines the CVoiceRecorder class, which encapsulates
// functionality for recording voice commands, finding the
// correct command, and persisting settings.
// Various supporting classes and definitions are also defined.
// This DLL is used by both the control panel extension and
// the background application.

#ifndef __VOICEREC_H
#define __VOICEREC_H

#include <f32file.h>
#include <s32strm.h>
#include <badesca.h>
#include <MdaAudioSampleEditor.h>

// uncomment this line to log the recognition process to a file
//#define DEBUG_RECOGNITION

// the CMdaAudioRecorderUtility state number which signifies an idle state
const int KRecorderIdle = 0;
// the CMdaAudioRecorderUtility state number which signifies that
// recording can commence
const int KReadyToRecord = 1;

// the maximum length, in characters, of a voice command name
const TInt KMaxVoiceCommandNameLength = 256;
// the maximum length, in characters, of a voice command argument
const TInt KMaxVoiceCommandArgumentLength = 256;
const TInt KMaxByte = 256;
// the maximum number of "signatures" which can be used to distinguish
// between commands
const TInt KMaxSignatures = 5;

// the factor used to equalise the sizes of voice command files
const TInt KSizeScalingFactor = 10000;

// The location of the voice control INI file. Note that this is
// always stored on the C: drive.
_LIT(KVoiceCommandFileLocation, "c:\\system\\data\\VoiceControl.ini");

// The UIDs of the various components of the system.
const TUid KUidVoiceControl = { 0x101F5589 };
const TUid KUidVoiceDaemon = { 0x101F558A };
const TUid KUidVoiceRec = { 0x101F558B };

// An enumeration which specifies IDs for the different commands
// which can be executed.
enum TVoiceCommandId
	{
	EVoiceCommandDialContact = 0,
	EVoiceCommandDialContactMobile,
	EVoiceCommandDialNumber,
	EVoiceCommandOpenDocument,
	EVoiceCommandOpenApplication
	};

// An enumeration which specifies the actions which can be performed
// after a telephone number is dialled.
enum TActionAfterDialling
	{
	EActionGoToBack = 0,
	EActionBringPhoneToFront,
	EActionUseInternalUi
	};

// This class defines a set of measures which can distinguish between
// two voice commands.
class TVoiceSignature
	{
public:
	// iLevel[x] is the number of bytes in a command file which equal x.
	TInt iLevel[KMaxByte];
	// iGradient[x] is the number of bytes in a command file whose
	// difference with the next byte equals x.
	TInt iGradient[KMaxByte];
	// the size of the command file
	TInt iFileSize;
	// the number of times the command file crosses the point of silence.
	TInt iCrossings;
	// the number of breaks in the command.
	TInt iGaps;
	};

// This class encapsulates a voice command.
class CVoiceCommand : public CBase
	{
public:
	// destroys resources used by the command
	IMPORT_C virtual ~CVoiceCommand();
	// the command name
	HBufC* iName;
	// the action to perform when the command is executed
	TVoiceCommandId iCommand;
	// the command argument
	HBufC* iArgument;
	// some distinguishing information about the command
	TVoiceSignature iSignature[KMaxSignatures];
	};

typedef RPointerArray<CVoiceCommand> RVoiceCommandArray;

// This class is used by both the control panel extension and
// the background application for all of their command-related
// functionality.
// In particular, it uses CMdaAudioRecorderUtility for all its
// audio-related needs.
class CVoiceRecorder : public CBase
	{
public:
	// Create a new voice recorder object. A reference to an existing
	// file session object is passed in to avoid creating a new session.
	IMPORT_C CVoiceRecorder(RFs& aFs);
	// Frees resources associated with the voice recorder.
	IMPORT_C ~CVoiceRecorder();

	// Creates a new file into which a voice command can be recorded.
	// The observer passed to this function will be called back when the
	// file is created and the audio recorder utility class is ready
	// to record.
	IMPORT_C void CreateVoiceFileL(const TFileName& aFileName,
		MMdaObjectStateChangeObserver& aObserver);
	// Actually starts recording into the previously specified file.
	IMPORT_C void RecordL();
	// Stops recording and closes the voice command file.
	IMPORT_C void Stop();

	// Opens a file containing raw audio data and removes leading and
	// trailing silences.
	IMPORT_C TInt CompactVoiceFileL(const TFileName& aOutputFile, const TFileName& aInputFile);
	// Finds the voice command in the array of voice commands which is
	// most like the one passed in.
	IMPORT_C CVoiceCommand* FindClosestVoiceCommandL(const CVoiceCommand* aCommand);

	// Saves the settings to a file.
	IMPORT_C void ExternalizeL();
	// Restores settings from a file.
	IMPORT_C void InternalizeL();
	// A utility function which places the names of all the voice commands into
	// an array - this is used to display the list of commands in the control
	// panel applet.
	IMPORT_C void GetVoiceCommandNamesL(CDesCArray& aNameArray);

	// Creates the signatures for a specified command based on the contents of
	// a specified file. When more than one repetition is being made of each
	// command, the index passed in specifies which repetition to modify.
	IMPORT_C void CreateSignatureL(const TFileName& aFileName, CVoiceCommand* aCommand, TInt aSignatureIndex);

	// Returns a reference to the voice command array.
	IMPORT_C RVoiceCommandArray& Commands();

	// Functions to enable or disable voice control.
	IMPORT_C TBool EnableVoiceControl();
	IMPORT_C void SetEnableVoiceControl(TBool aEnableVoiceControl);

	// Functions to enable or disable the playing of a tone when a command
	// is recorded.
	IMPORT_C TBool EnablePlayTone();
	IMPORT_C void SetEnablePlayTone(TBool aEnablePlayTone);

	// Functions to get/set the number of times each command is recorded.
	IMPORT_C TInt RecordRepetitions();
	IMPORT_C void SetRecordRepetitions(TInt aRecordRepetitions);

	// Functions to get/set the action to be performed after dialling
	// a telephone number.
	IMPORT_C TActionAfterDialling ActionAfterDialling();
	IMPORT_C void SetActionAfterDialling(TActionAfterDialling aActionAfterDialling);

	// Functions to get/set the hot key to be used to invoke the voice
	// control background application.
	IMPORT_C void GetHotKey(TInt& aScanCode, TUint& aModifiers);
	IMPORT_C void SetHotKey(TInt aScanCode, TUint aModifiers);

private:
	// Helper function which returns whether the byte passed in represents
	// silence in raw, signed 8-bit audio format.
	TBool IsSilence(TUint8 aByte);

	// The recorder utility object used for all audio operations.
	CMdaAudioRecorderUtility *iRecorder;
	// The type of the voice file used - always 8-bit, unsigned raw.
	CMdaAudioType* iVoiceFileType;
	// The filename of the voice file. This is constant and specified at the
	// top of the file.
	TMdaFileClipLocation iVoiceFileLocation;
	// A reference to the file server session.
	RFs& iFs;
	// The letter of the drive this DLL is stored on.
	TBuf<10> iDriveLetter;

	// The array of voice commands.
	RVoiceCommandArray iCommandArray;

	// Variables recording the voice control settings currently in use.
	TBool iEnableVoiceControl;
	TBool iEnablePlayTone;
	TInt iRecordRepetitions;
	TActionAfterDialling iActionAfterDialling;
	TInt iHotKeyScanCode;
	TUint iHotKeyModifiers;
	};

#endif