#include <s32file.h>

#include "VoiceRec.h"

// This is the main entry point for the DLL.
GLDEF_C TInt E32Dll(TDllReason /*aReason*/) 
    { 
    return KErrNone;
    }

//--------------------------------------------------------------

// The destructor of the CVoiceCommand class. It frees up any memory
// which may have been allocated.
EXPORT_C CVoiceCommand::~CVoiceCommand()
	{
	delete iName;
	delete iArgument;
	}

//--------------------------------------------------------------

// The constructor for the CVoiceRecorder class. Its only purpose is to
// initialise the drive name to be used for file access later on. Note that
// the class is passed in a reference to an already-existing file server
// session for efficiency.
EXPORT_C CVoiceRecorder::CVoiceRecorder(RFs& aFs) : iFs(aFs)
	{
	// Get the filename of this DLL and initialise a TParse object with it.
	TParse parse;
	TFileName tempFileName;
	Dll::FileName(tempFileName);
	parse.SetNoWild(tempFileName, NULL, NULL);
	// We need to set the default path to be used for future file access, because
	// it is always initialised to c: and the voice control files may be stored
	// on the d: drive.
	iDriveLetter.Copy(parse.Drive());
	TFileName drivePath = iDriveLetter;
	_LIT(KBackslash, "\\");
	drivePath.Append(KBackslash);
	iFs.SetSessionPath(drivePath);
	}

// This function creates a new file to which the audio data which is
// recorded will be written. The voice recorder object is initialised
// with sensible settings.
EXPORT_C void CVoiceRecorder::CreateVoiceFileL(const TFileName& aFileName,
	MMdaObjectStateChangeObserver& aObserver)
	{
	// delete the current file
	iFs.Delete(aFileName);
	
	// delete the voice recorder object, if it exists
	if (iRecorder)
		{
		delete iRecorder;
		iRecorder = NULL;
		}

	// delete the voice file type object, if it exists
	if (iVoiceFileType)
		{
		delete iVoiceFileType;
		iVoiceFileType = NULL;
		}

	// actually open the recorder
	iRecorder = CMdaAudioRecorderUtility::NewL(aObserver);

	// The file location must be specified as an absolute path, so we need
	// to include the drive letter. We assume that it's passed in as a relative
	// path.
	iVoiceFileLocation.iName = iDriveLetter;
	iVoiceFileLocation.iName.Append(aFileName);

	// We record voice commands using RAW audio format, of the lowest
	// possible quality, in order to reduce disk space and make file
	// comparisons easier.
	iVoiceFileType = new (ELeave) CMdaAudioType;
	iVoiceFileType->iFormat = new (ELeave) TMdaRawAudioClipFormat;
	// Use unsigned 8-bit raw audio. This is the easiest format to use.
	iVoiceFileType->iCodec = new (ELeave) TMdaU8PcmRawAudioCodec;
	iVoiceFileType->iSettings = new (ELeave) TMdaAudioDataSettings;
	
	// If a sample rate of 32000 is specified, recording stops after ~1s.
	// 16000 works but file sizes are too large.
	iVoiceFileType->iSettings->iSampleRate = 8000;
	
	// Mono must be specified to avoid a panic.
	iVoiceFileType->iSettings->iChannels = 1;
	
	// Actually open the audio recorder object, using the previously
	// specified settings.
	iRecorder->OpenL(&iVoiceFileLocation, iVoiceFileType->iFormat, iVoiceFileType->iCodec,
		iVoiceFileType->iSettings);
	}

EXPORT_C void CVoiceRecorder::RecordL()
	{
	// Setting the volume doesn't seem to do much - setting the gain
	// to max (32767 on the 9210) does improve quality of recording though.
	iRecorder->SetGain(iRecorder->MaxGain());

	// Actually start recording.
	iRecorder->RecordL();
	}

EXPORT_C void CVoiceRecorder::Stop()
	{
	// Stop the recording and close the audio recorder.
	iRecorder->Stop();
	iRecorder->Close();
	}

// A simple helper function which determines whether the byte passed in
// represents silence or a sound. As the format used is raw, unsigned 8-bit
// data, a value of 0x80 represents complete silence. This function
// (arbitrarily) assumes anything close to 0x80 is silence too.
TBool CVoiceRecorder::IsSilence(TUint8 aByte)
	{
	return ((aByte >= 0x78) && (aByte <= 0x88));
	}

// This function opens a file of raw audio data and compresses it. It
// achieves this by removing the periods of silence that occur at the
// beginning and end. In the case of voice commands, this can result in
// a big space saving.
EXPORT_C TInt CVoiceRecorder::CompactVoiceFileL(const TFileName& aOutputFile,
												const TFileName& aInputFile)
	{
	// Open the input file.
	RFile file;
	User::LeaveIfError(file.Open(iFs, aInputFile, EFileRead | EFileShareAny));
	CleanupClosePushL(file);
	
	// Get the file's size.
	TInt fileSize;
	User::LeaveIfError(file.Size(fileSize));
	
	// Allocate a buffer big enough to hold the file, and get a writable
	// pointer to it. Note that this is a lazy approach - the file does not
	// have to be completely loaded into memory.
	HBufC8* buf = HBufC8::NewLC(fileSize);
	TPtr8 bufPtr = buf->Des();
	
	// Read the file into memory and close it.
	User::LeaveIfError(file.Read(bufPtr));

	// Close the file object. Note that we have to remove the buffer from
	// the cleanup stack and put it back again.
	CleanupStack::Pop(); // buf
	CleanupStack::PopAndDestroy(); // file
	CleanupStack::PushL(buf);

	TInt startPos, endPos;

	// Skip silence at the start of the file.
	// Always skip a few bytes as well (there's usually some junk at the start).
	for (startPos = 0; (startPos < fileSize) &&
		(IsSilence(bufPtr[startPos]) || (startPos < 0x1000)) ; startPos++)
		;
	
	// If we've reached the end of the file and we still haven't found
	// anything that's not silence, return an error. Note that we don't leave
	// here, so that the caller can distinguish this error from a more serious
	// one.
	if (startPos == fileSize)
		{
		CleanupStack::PopAndDestroy(); // buf
		return KErrNotFound;
		}
	
	// Skip silence at the end of the file. There usually isn't any trailing
	// junk like at the start.
	for (endPos = fileSize - 1; (endPos > startPos) &&
		IsSilence(bufPtr[endPos]); endPos--)
		;
	
	// Open the output file.
	RFile outputFile;
	User::LeaveIfError(outputFile.Replace(iFs, aOutputFile,
		EFileWrite | EFileShareExclusive));
	CleanupClosePushL(outputFile);
	
	// Write the compressed audio data to the file.
	TPtrC8 voicePtr = bufPtr.Mid(startPos, endPos - startPos);
	User::LeaveIfError(outputFile.Write(voicePtr));
	
	CleanupStack::PopAndDestroy(2); // buf, outputFile

	return KErrNone;
	}

// This function determines which command, in the array of voice commands,
// is closest to the command it's passed. Its algorithm for doing this to
// calculate the total difference in each signature between each command in the
// array, and the one it was passed. It then calculates which command has the
// largest number of signatures closest to the one passed in, and returns it.
EXPORT_C CVoiceCommand* CVoiceRecorder::FindClosestVoiceCommandL(const CVoiceCommand* aCommand)
	{
	TInt commandCount = iCommandArray.Count();

	// Don't even bother searching if there aren't any commands in the array.
	if (commandCount == 0)
		return NULL;

	// The minimum differences in each signature.
	TInt minLevelDiff = KMaxTInt;
	TInt minGradientDiff = KMaxTInt;
	TInt minSizeDiff = KMaxTInt;
	TInt minCrossingsDiff = KMaxTInt;
	TInt minGapsDiff = KMaxTInt;

	// The command that's the most likely candidate for each signature.
	CVoiceCommand* bestFitLevel = NULL;
	CVoiceCommand* bestFitGradient = NULL;
	CVoiceCommand* bestFitSize = NULL;
	CVoiceCommand* bestFitCrossings = NULL;
	CVoiceCommand* bestFitGaps = NULL;

	// For the current command in the array, the differences between it and
	// the command passed in.
	TInt levelDiff = 0;
	TInt gradientDiff = 0;
	TInt sizeDiff = 0;
	TInt crossingsDiff = 0;
	TInt gapsDiff = 0;

	TInt commandIndex;
	TInt sig;
	for (commandIndex = 0; commandIndex < commandCount; commandIndex++)
		{
		levelDiff = gradientDiff = sizeDiff = crossingsDiff = gapsDiff = 0;

		for (sig = 0; sig < iRecordRepetitions; sig++)
			{
			for (TInt i = 0; i < KMaxByte; i++)
				{
				// Calculate the differences in level between this command
				// and the one we were passed in.
				if (aCommand->iSignature[0].iLevel[i] <
					iCommandArray[commandIndex]->iSignature[sig].iLevel[i])
					{
					levelDiff += (iCommandArray[commandIndex]->iSignature[sig].iLevel[i] -
						aCommand->iSignature[0].iLevel[i]);
					}
				else
					{
					levelDiff += (aCommand->iSignature[0].iLevel[i] -
						iCommandArray[commandIndex]->iSignature[sig].iLevel[i]);
					}
				
				// Calculate the differences in gradient between this command
				// and the one we were passed in.
				if (aCommand->iSignature[0].iGradient[i] <
					iCommandArray[commandIndex]->iSignature[sig].iGradient[i])
					{
					gradientDiff += (iCommandArray[commandIndex]->iSignature[sig].iGradient[i] -
						aCommand->iSignature[0].iGradient[i]);
					}
				else
					{
					gradientDiff += (aCommand->iSignature[0].iGradient[i] -
						iCommandArray[commandIndex]->iSignature[sig].iGradient[i]);
					}
				}
			
			// Calculate the difference in file size between this command and
			// the one we were passed in.
			if (aCommand->iSignature[0].iFileSize < iCommandArray[commandIndex]->iSignature[sig].iFileSize)
				sizeDiff += iCommandArray[commandIndex]->iSignature[sig].iFileSize - aCommand->iSignature[0].iFileSize;
			else
				sizeDiff += aCommand->iSignature[0].iFileSize - iCommandArray[commandIndex]->iSignature[sig].iFileSize;
			
			// Calculate the difference in the number of crossings between
			// this command and the one we were passed in.
			if (aCommand->iSignature[0].iCrossings < iCommandArray[commandIndex]->iSignature[sig].iCrossings)
				crossingsDiff += iCommandArray[commandIndex]->iSignature[sig].iCrossings - aCommand->iSignature[0].iCrossings;
			else
				crossingsDiff += aCommand->iSignature[0].iCrossings - iCommandArray[commandIndex]->iSignature[sig].iCrossings;
			
			// Calculate the difference in the number of gaps between this
			// command and the one we were passed in.
			if (aCommand->iSignature[0].iGaps < iCommandArray[commandIndex]->iSignature[sig].iGaps)
				gapsDiff += iCommandArray[commandIndex]->iSignature[sig].iGaps - aCommand->iSignature[0].iGaps;
			else
				gapsDiff += aCommand->iSignature[0].iGaps - iCommandArray[commandIndex]->iSignature[sig].iGaps;
			}
		
		// If the command is the most likely so far in any of the signatures,
		// record this fact.
		if (levelDiff < minLevelDiff)
			{
			bestFitLevel = iCommandArray[commandIndex];
			minLevelDiff = levelDiff;
			}
		if (gradientDiff < minGradientDiff)
			{
			bestFitGradient = iCommandArray[commandIndex];
			minGradientDiff = gradientDiff;
			}
		if (sizeDiff < minSizeDiff)
			{
			bestFitSize = iCommandArray[commandIndex];
			minSizeDiff = sizeDiff;
			}
		if (crossingsDiff < minCrossingsDiff)
			{
			bestFitCrossings = iCommandArray[commandIndex];
			minCrossingsDiff = crossingsDiff;
			}
		if (gapsDiff < minGapsDiff)
			{
			bestFitGaps = iCommandArray[commandIndex];
			minGapsDiff = gapsDiff;
			}
		}

	// Define "DEBUG_RECOGNITION" to turn on logging of the recognition process.
	// This can be useful if you want to modify it or see how it works.

#ifdef DEBUG_RECOGNITION

	_LIT(KVoiceLogFileName, "c:\\voicelog.txt");

	RFile logFile;
	TInt seek = 0;
	TInt err = logFile.Open(iFs, KVoiceLogFileName, EFileWrite | EFileShareAny);
	if (err == KErrNotFound)
		logFile.Create(iFs, KVoiceLogFileName, EFileWrite | EFileShareAny);
	else
		logFile.Seek(ESeekEnd, seek);
	CleanupClosePushL(logFile);

	TBuf8<256> logBuf;
	if (bestFitLevel != NULL)
		{
		TBuf8<256> buf8;
		buf8.Copy(*(bestFitLevel->iName));
		_LIT8(KLevelFormat, "\r\nClosest in level: %S\r\n");
		logBuf.Format(KLevelFormat, &buf8);
		logFile.Write(logBuf);
		}
	if (bestFitGradient != NULL)
		{
		TBuf8<256> buf8;
		buf8.Copy(*(bestFitGradient->iName));
		_LIT8(KGradientFormat, "\r\nClosest in gradient: %S * 2\r\n");
		logBuf.Format(KGradientFormat, &buf8);
		logFile.Write(logBuf);
		}
	if (bestFitSize != NULL)
		{
		TBuf8<256> buf8;
		buf8.Copy(*(bestFitSize->iName));
		_LIT8(KSizeFormat, "\r\nClosest in size: %S\r\n");
		logBuf.Format(KSizeFormat, &buf8);
		logFile.Write(logBuf);
		}
	if (bestFitCrossings != NULL)
		{
		TBuf8<256> buf8;
		buf8.Copy(*(bestFitCrossings->iName));
		_LIT8(KCrossingsFormat, "\r\nClosest in crossings: %S\r\n");
		logBuf.Format(KCrossingsFormat, &buf8);
		logFile.Write(logBuf);
		}
	if (bestFitGaps != NULL)
		{
		TBuf8<256> buf8;
		buf8.Copy(*(bestFitGaps->iName));
		_LIT8(KGapsFormat, "\r\nClosest in gaps: %S\r\n");
		logBuf.Format(KGapsFormat, &buf8);
		logFile.Write(logBuf);
		}
#endif

	// Create a "votes" variable for each command, to store the number
	// of signatures each command is the closest in.
	TInt* votes = new TInt[commandCount];
	for (TInt v = 0; v < commandCount; v++)
		votes[v] = 0;

	// Figure out how many votes each command has.
	votes[iCommandArray.Find(bestFitLevel)]++;
	// The gradient seems to be the best measure, so we give it two votes.
	votes[iCommandArray.Find(bestFitGradient)] += 2;
	votes[iCommandArray.Find(bestFitSize)]++;
	votes[iCommandArray.Find(bestFitCrossings)]++;
	votes[iCommandArray.Find(bestFitGaps)]++;
	
	// We want to return the command with the most votes.
	TInt maxVotes = 0;
	CVoiceCommand* returnCommand = NULL;
	for (commandIndex = 0; commandIndex < commandCount; commandIndex++)
		{
		if (votes[commandIndex] > maxVotes)
			{
			maxVotes = votes[commandIndex];
			returnCommand = iCommandArray[commandIndex];
			}
		}

	// Uncomment these lines to turn on logging of the recognition process.

#ifdef DEBUG_RECOGNITION
	TBuf8<256> nbuf8;
	nbuf8.Copy(*(returnCommand->iName));
	_LIT8(KClosestCommandFormat, "\r\nClosest command: %S\r\n");
	logBuf.Format(KClosestCommandFormat, &nbuf8);
	logFile.Write(logBuf);
	CleanupStack::PopAndDestroy(); // logFile
#endif

	return returnCommand;
	}

// This function creates "signatures" for a command - that is, values
// representing a voice command's properties.
EXPORT_C void CVoiceRecorder::CreateSignatureL(const TFileName& aFileName, CVoiceCommand* aCommand, TInt aSignatureIndex)
	{
	// Open the voice command's audio data file.
	RFile file;
	User::LeaveIfError(file.Open(iFs, aFileName, EFileRead | EFileShareAny));
	CleanupClosePushL(file);

	// Get the file's size and set the "file size" signature based on it.
	TInt fileSize;
	User::LeaveIfError(file.Size(fileSize));
	aCommand->iSignature[aSignatureIndex].iFileSize = fileSize;

	// Read the entire file into a newly allocated buffer.
	HBufC8* buf = HBufC8::NewLC(fileSize);
	TPtr8 bufPtr = buf->Des();
	User::LeaveIfError(file.Read(bufPtr));

	CleanupStack::Pop(); // buf
	CleanupStack::PopAndDestroy(); // file
	CleanupStack::PushL(buf);

	TInt i;
	
	// Reset each of the signatures before we start.
	aCommand->iSignature[aSignatureIndex].iCrossings = 0;
	aCommand->iSignature[aSignatureIndex].iGaps = 0;
	for (i = 0; i < 256; i++)
		{
		aCommand->iSignature[aSignatureIndex].iLevel[i] = 0;
		aCommand->iSignature[aSignatureIndex].iGradient[i] = 0;
		}

	for (i = 0; i < buf->Length(); i++)
		{
		if (i > 0)
			{
			TInt diff;
			// Calculate the difference between each byte and its
			// predecessor. As this ranges from 0-512, we divide it by
			// two to be in the same range as the "level" measure.
			if ((*buf)[i] > (*buf)[i-1])
				diff = 0x80 + (((*buf)[i] - (*buf)[i-1]) / 2);
			else
				diff = 0x80 - (((*buf)[i-1] - (*buf)[i]) / 2);
			aCommand->iSignature[aSignatureIndex].iGradient[diff]++;
			
			// If the audio signal has crossed the central mark, incrememnt
			// the number of crossings.
			if ((((*buf)[i] > 0x80) && ((*buf)[i-1] < 0x80)) ||
				(((*buf)[i] < 0x80) && ((*buf)[i-1] > 0x80)))
				aCommand->iSignature[aSignatureIndex].iCrossings++;
			
			// assume a "gap" is two or more silent bytes
			if (IsSilence((*buf)[i]) && IsSilence((*buf)[i-1]))
				aCommand->iSignature[aSignatureIndex].iGaps++;
			}
		aCommand->iSignature[aSignatureIndex].iLevel[(*buf)[i]]++;
		}

	CleanupStack::PopAndDestroy(); // buf

	// We now scale all of the measures (other than file size) by the file
	// size, in order to get more accurate comparisons between files of
	// different sizes.
	for (i = 0; i < 256; i++)
		{
		aCommand->iSignature[aSignatureIndex].iLevel[i] =
			(KSizeScalingFactor * aCommand->iSignature[aSignatureIndex].iLevel[i]) / fileSize;

		aCommand->iSignature[aSignatureIndex].iGradient[i] =
			(KSizeScalingFactor * aCommand->iSignature[aSignatureIndex].iGradient[i]) / fileSize;
		}
	aCommand->iSignature[aSignatureIndex].iCrossings =
		(KSizeScalingFactor * aCommand->iSignature[aSignatureIndex].iCrossings) / fileSize;
	aCommand->iSignature[aSignatureIndex].iGaps =
		(KSizeScalingFactor * aCommand->iSignature[aSignatureIndex].iGaps) / fileSize;
	}

// The destructor just frees all the class' associated memory.
EXPORT_C CVoiceRecorder::~CVoiceRecorder()
	{
	iCommandArray.ResetAndDestroy();
	delete iRecorder;
	delete iVoiceFileType;
	}

// This function outputs all of the voice recorder's settings and all the
// voice commands to the standard INI file location.
EXPORT_C void CVoiceRecorder::ExternalizeL()
	{
	// Open the INI file.
	RFileWriteStream stream;
	User::LeaveIfError(stream.Replace(iFs, KVoiceCommandFileLocation,
		EFileWrite | EFileShareAny));
	CleanupClosePushL(stream);

	// Write all the settings.
	stream.WriteInt32L(iEnableVoiceControl);
	stream.WriteInt32L(iEnablePlayTone);
	stream.WriteInt32L(iRecordRepetitions);
	stream.WriteInt32L(iActionAfterDialling);
	stream.WriteInt32L(iHotKeyScanCode);
	stream.WriteUint32L(iHotKeyModifiers);

	stream.WriteInt32L(iCommandArray.Count());
	
	// Write each of the commands and their signatures.
	for (TInt i = 0; i < iCommandArray.Count(); i++)
		{
		stream << *(iCommandArray[i]->iName);
		stream.WriteInt32L(iCommandArray[i]->iCommand);
		stream << *(iCommandArray[i]->iArgument);

		for (TInt sig = 0; sig < KMaxSignatures; sig++)
			{
			TInt j;
			for (j = 0; j < KMaxByte; j++)
				stream.WriteUint8L(iCommandArray[i]->iSignature[sig].iLevel[j]);
			for (j = 0; j < KMaxByte; j++)
				stream.WriteUint8L(iCommandArray[i]->iSignature[sig].iGradient[j]);
			stream.WriteInt32L(iCommandArray[i]->iSignature[sig].iFileSize);
			stream.WriteInt32L(iCommandArray[i]->iSignature[sig].iCrossings);
			stream.WriteInt32L(iCommandArray[i]->iSignature[sig].iGaps);
			}
		}
	CleanupStack::PopAndDestroy();
	}

// This function reads all the voice recorder's settings and voice commands
// from the standard INI file location.
EXPORT_C void CVoiceRecorder::InternalizeL()
	{
	TInt count = 0;
	// Open the INI file for reading.
	RFileReadStream stream;
	TInt rc = stream.Open(iFs, KVoiceCommandFileLocation, EFileRead | EFileShareAny);

	if (rc == KErrNone)
		{
		CleanupClosePushL(stream);

		// Read all of the settings from the file.
		iEnableVoiceControl = stream.ReadInt32L();
		iEnablePlayTone = stream.ReadInt32L();
		iRecordRepetitions = stream.ReadInt32L();
		iActionAfterDialling = (TActionAfterDialling) stream.ReadInt32L();
		iHotKeyScanCode = stream.ReadInt32L();
		iHotKeyModifiers = stream.ReadUint32L();

		// Read the voice commands.
		count = stream.ReadInt32L();
		for (TInt i = 0; i < count; i++)
			{
			CVoiceCommand* cmd = new (ELeave) CVoiceCommand;
			cmd->iName = HBufC::NewL(stream, KMaxVoiceCommandNameLength);
			cmd->iCommand = (TVoiceCommandId) stream.ReadInt32L();
			cmd->iArgument = HBufC::NewL(stream, KMaxVoiceCommandArgumentLength);

			for (TInt sig = 0; sig < KMaxSignatures; sig++)
				{
				TInt j;
				for (j = 0; j < KMaxByte; j++)
					cmd->iSignature[sig].iLevel[j] = stream.ReadUint8L();
				for (j = 0; j < KMaxByte; j++)
					cmd->iSignature[sig].iGradient[j] = stream.ReadUint8L();

				cmd->iSignature[sig].iFileSize = stream.ReadInt32L();
				cmd->iSignature[sig].iCrossings = stream.ReadInt32L();
				cmd->iSignature[sig].iGaps = stream.ReadInt32L();
				}
			iCommandArray.Append(cmd);
			}
		CleanupStack::PopAndDestroy();
		}
	else
		{
		// If we couldn't open the INI file, create some default settings.
		iRecordRepetitions = 1;
		iEnableVoiceControl = ETrue;
		iEnablePlayTone = ETrue;
		iActionAfterDialling = EActionGoToBack;
		iHotKeyScanCode = EStdKeyMenu;
		iHotKeyModifiers = EModifierCtrl;
		}
	}

// This is a simple helper function to get the names of all the voice
// commands into an array.
EXPORT_C void CVoiceRecorder::GetVoiceCommandNamesL(CDesCArray& aNameArray)
	{
	aNameArray.Reset();
	for (TInt i = 0; i < iCommandArray.Count(); i++)
		{
		aNameArray.AppendL(*(iCommandArray[i]->iName));
		}
	}

// Accessor function that returns a reference to the voice command array.
EXPORT_C RVoiceCommandArray& CVoiceRecorder::Commands()
	{
	return iCommandArray;
	}

// Setter and getter functions providing access to Voice Control settings.

EXPORT_C TBool CVoiceRecorder::EnableVoiceControl()
	{
	return iEnableVoiceControl;
	}

EXPORT_C void CVoiceRecorder::SetEnableVoiceControl(TBool aEnableVoiceControl)
	{
	iEnableVoiceControl = aEnableVoiceControl;
	}

EXPORT_C TBool CVoiceRecorder::EnablePlayTone()
	{
	return iEnablePlayTone;
	}

EXPORT_C void CVoiceRecorder::SetEnablePlayTone(TBool aEnablePlayTone)
	{
	iEnablePlayTone = aEnablePlayTone;
	}

EXPORT_C TInt CVoiceRecorder::RecordRepetitions()
	{
	return iRecordRepetitions;
	}

EXPORT_C void CVoiceRecorder::SetRecordRepetitions(TInt aRecordRepetitions)
	{
	iRecordRepetitions = aRecordRepetitions;
	}

EXPORT_C TActionAfterDialling CVoiceRecorder::ActionAfterDialling()
	{
	return iActionAfterDialling;
	}

EXPORT_C void CVoiceRecorder::SetActionAfterDialling(TActionAfterDialling aActionAfterDialling)
	{
	iActionAfterDialling = aActionAfterDialling;
	}

EXPORT_C void CVoiceRecorder::GetHotKey(TInt& aScanCode, TUint& aModifiers)
	{
	aScanCode = iHotKeyScanCode;
	aModifiers = iHotKeyModifiers;
	}

EXPORT_C void CVoiceRecorder::SetHotKey(TInt aScanCode, TUint aModifiers)
	{
	iHotKeyScanCode = aScanCode;
	iHotKeyModifiers = aModifiers;
	}